% The similarity matrix is computed using the multi-scale distortion
% defined in:
%
% % "A Game-Theoretic Approach to Deformable Shape Matching"
% E.Rodola, A.Bronstein, A.Albarelli, F.Bergamasco, and A.Torsello
% 2012 IEEE Intl. Conf. on Computer Vision and Pat. Recog. (CVPR 2012)
%
% Please cite the paper above if you use this function.

function P = sim_matrix(shape1, shape2, candidate_matches)

m1 = candidate_matches(:,1);
m2 = candidate_matches(:,2);

Q = 0;
diffDistTimes = 2.^(7:16);

fprintf('Computing similarity matrix... ');

for t=1:numel(diffDistTimes),
    
    % shape2
    [unqIdx,~,unqReverse] = unique(m2);
    [Ds,tr] = DiffusionDistanceMatrix(shape2.evecs(unqIdx,:), shape2.evals, diffDistTimes(t));
    Ds = Ds(unqReverse,unqReverse) / (tr^2);
    
    % shape1
    [unqIdx,~,unqReverse] = unique(m1);
    [Dn,tr] = DiffusionDistanceMatrix(shape1.evecs(unqIdx,:), shape1.evals, diffDistTimes(t));
    Dn = Dn(unqReverse,unqReverse) / (tr^2);
    
    Q = Q + (sqrt(Dn)-sqrt(Ds)).^2;
end
clear unqIdx unqReverse Dn Ds

Q = exp(-Q);

[I,J] = meshgrid(1:length(candidate_matches), 1:length(candidate_matches));
NotBiject = reshape( (candidate_matches(I,1) == candidate_matches(J,1) | candidate_matches(I,2) == candidate_matches(J,2)), size(I) );
Q(NotBiject) = 0;
clear I J NotBiject

% NOTE: you might want to change the exponent below depending on the
% application or the shape class

P = Q.^2e6;
fprintf('done\n');

end
